/**
 * @file      bmeipc.h: BME IPC definitions
 *
 * @author    <mikko.k.ylinen at nokia.com>
 *
 * @modified  <aliaksei.katovich at nokia.com>
 * @modified  <matti.halme at nokia.com>
 *
 * @date      Created: Thu Jul 30 09:19:59 EEST 2009
 *
 * Copyright (C) 2009 - 2011 Nokia. All rights reserved.
 *
 */

#ifndef __BMEIPC_H__
#define __BMEIPC_H__

#include <stdint.h>
#include <sys/time.h>

#define BMEIPC_TIMEOUT		5000	/* ms */

#define BMEVENT_NONE		0L
#define BMEVENT_ERROR		(1L << 0)
#define BMEVENT_TIMEOUT		(1L << 1)
#define BMEVENT_EMHAL		(1L << 2)
#define BMEVENT_GENERIC		(1L << 3)
#define BMEVENT_SYSTEM		(1L << 4)
#define BMEVENT_CHARGE		(1L << 5)
#define BMEVENT_CHARGER		(1L << 6)
#define BMEVENT_BATMON		(1L << 7)
#define BMEVENT_PSM		(1L << 8)
#define BMEVENT_ADC		(1L << 9)
#define BMEVENT_THERMO		(1L << 10)

#define BMEIPC_MQMEAS		"/em_measmq"	/* measurements message queue */
#define BMEIPC_MQMEAS_MAX	128		/* number of messages in queue */

/* FIXME: BMEIPC_MQNAME is kept for backward compatibility */
#define BMEIPC_MQNAME		BMEIPC_MQMEAS

/* -------------------- basic structures -------------------- */

typedef int32_t (*bmeipc_cb_t) (int32_t); /* BME IPC callback function type */

#pragma pack(push)
#pragma pack(4)

#define BME_IPC_MSG_MAX_SIZE (0x80)

/** Generic message */
typedef struct bmeipc_msg_s {
	uint16_t type;
	uint16_t subtype;
} bmeipc_msg_t;

/** Server PID reply */
typedef struct bmeipc_pid_s {
	uint32_t zero;
	uint32_t pid;
} bmeipc_pid_t;

/** measurements message */
typedef struct bmeipc_meas_s {
	uint8_t	state;			/* bmeipc_meas_state_e */
	struct  timeval timestamp;
	int16_t bat_current;
	int16_t bat_temp;
	int16_t bat_voltage;
} bmeipc_meas_t;

#pragma pack(pop)

/* --------------------------------------------------------------------
 *
 * Constants below are enumerated explicitly to keep BME API consistent
 *
 * --------------------------------------------------------------------*/

enum bmeipc_meas_state_e {
	MEASUREMENTS_OFF		= 0,
	MEASUREMENTS_RUN		= 1,
	MEASUREMENTS_ERROR		= 0xff,
};

enum bmeipc_battery_state_e {
	BATTERY_STATE_EMPTY		= 0,
	BATTERY_STATE_LOW		= 1,
	BATTERY_STATE_OK		= 2,
	BATTERY_STATE_FULL		= 3,
	BATTERY_STATE_ERROR		= 0xff,
};

enum bmeipc_battery_type_e {
	BATTERY_TYPE_NONE		= 0,
	BATTERY_TYPE_SERVICE		= 1,
	BATTERY_TYPE_NICKEL		= 2,
	BATTERY_TYPE_LI4V1		= 3,
	BATTERY_TYPE_LI4V2		= 4,
	BATTERY_TYPE_LI4V35		= 5,
	BATTERY_TYPE_ERROR		= 0xff,
};

enum bmeipc_charger_state_e {
	CHARGER_STATE_DISCONNECTED	= 0,
	CHARGER_STATE_CONNECTED		= 1,
	CHARGER_STATE_ERROR		= 0xff,
};

enum bmeipc_charger_type_e {
	CHARGER_TYPE_NONE		= 0,
	CHARGER_TYPE_USB100MA		= 1,
	CHARGER_TYPE_USB500MA		= 2,
	CHARGER_TYPE_USBWALL		= 3,
	CHARGER_TYPE_DYNAMO		= 4,
	CHARGER_TYPE_USB900MA		= 5,
	CHARGER_TYPE_ERROR		= 0xff,
};

enum bmeipc_charging_state_e {
	CHARGING_STATE_STOPPED		= 0,
	CHARGING_STATE_STARTED		= 1,
	CHARGING_STATE_SPECIAL		= 2,	/* e.g. maintenance charging */
	CHARGING_STATE_ERROR		= 0xff,
};

enum bmeipc_charging_type_e {
	CHARGING_TYPE_NONE		= 0,
	CHARGING_TYPE_LITHIUM		= 1,
	CHARGING_TYPE_ERROR		= 0xff,
};

enum bmeipc_battery_condition_e {
	BATTERY_CONDITION_GOOD		= 0,
	BATTERY_CONDITION_POOR		= 1,
	BATTERY_CONDITION_UNKNOWN	= 0xff,
};

enum bmeipc_bmestat_index_e {
	BMESTAT_FLAGS			= 0,
	CHARGER_STATE			= 1,	/* bmeipc_charger_state_e */
	CHARGER_TYPE			= 2,	/* bmeipc_charger_type_e */
	CHARGING_STATE			= 3,	/* bmeipc_charging_state_e */
	CHARGING_TYPE			= 4,	/* bmeipc_charging_type_e */
	CHARGING_TIME			= 5,	/* charging time left [min] */
	BATTERY_STATE			= 6,	/* bmeipc_battery_state_e */
	BATTERY_TYPE			= 7,	/* bmeipc_battery_type_e */
	BATTERY_LEVEL_MAX		= 8,	/* bars */
	BATTERY_LEVEL_NOW		= 9,	/* bars */
	BATTERY_TIME_IDLE		= 10,	/* [h] (legacy, obsolete) */
	BATTERY_TIME_LEFT		= 11,	/* [minutes] (legacy, obs.) */
	BATTERY_CAPA_MAX		= 12,	/* designed max capacity [mAh ] */
	BATTERY_CAPA_NOW		= 13,	/* remaining capacity[mAh ] */
	BATTERY_VOLT_MAX		= 14,	/* designed voltage  [mV] */
	BATTERY_VOLT_NOW		= 15,	/* current voltage   [mV] */
	BATTERY_LEVEL_PCT		= 16,	/* remaining capacity [%] */
	BATTERY_TEMP			= 17,	/* [Kelvin] */
	BATTERY_CURRENT                 = 18,   /* battery current mA */
	COULOMB_COUNTER                 = 19,   /* coulomb counter mAs */
	BATTERY_CAPA_FULL               = 20,   /* The capacity at full */
	SYSTEM_STATE			= 21,	/* e.g. acting dead etc. */
	BATTERY_CONDITION               = 22,   /* bmeipc_battery_cond_e */
	BMESTAT_MAX			= 32,	/* xx - 31 reserved */
};

typedef int32_t bmestat_t[BMESTAT_MAX];

/* -------------------- generic routines -------------------- */

/**
 * Configure library according to environment variables if any
 */
void bmeipc_getenv(void);

/**
 * Get energy management statistics
 *
 * @param sd socket descriptor
 * @param stat statistics to return
 *
 * @return:	>=0 on success, -1 on error
 */
int32_t bmeipc_stat(int32_t sd, bmestat_t *stat);

/* -------------------- socket I/O primitives -------------------- */

/**
 * Poll socket
 *
 * @param sd socket descriptor
 * @param evt event to poll
 * @param tout polling timeout
 *
 * @return	>= 0 on success, -1 on error
 */
int32_t bmeipc_poll(int32_t sd, int16_t evt, int32_t tout);

/**
 * Receive BME data from socket
 *
 * @param sd socket descriptor
 * @param msg data address
 * @param bytes	data size
 *
 * @return number of bytes read, -1 on error
 *
 * @ingroup bmeipc
 */
int32_t bmeipc_recv(int32_t sd, void *data, int32_t bytes);
/* _deprecated_ variant */
int32_t bme_packet_read(int32_t sd, void *data, int32_t bytes);

/**
 * Send BME data to socket.
 *
 * @param sd socket descriptor
 * @param msg data address
 * @param bytes data size
 *
 * @return	number of bytes send, -1 on error
 *
 * @credential  bme::BatteryControl  Needed for all non read-only messages
 *
 * @ingroup bmeipc
 */
int32_t bmeipc_send(int32_t sd, const void *data, int32_t bytes);
/* _deprecated_ variant */
int32_t bme_packet_write(int32_t sd, const void *data, int32_t bytes);

/**
 * Close connection to BME server
 *
 * @ingroup bmeipc
 */
void	bmeipc_close(int32_t sd);
/* _deprecated_ non-reentrant variant */
void	bme_disconnect(void);

/**
 * Open connection to BME server
 *
 * @ingroup bmeipc
 *
 * @return socket descriptor on success, -1 on error
 */
int32_t bmeipc_open(void);
/* _deprecated_ non-reentrant variant */
int32_t bme_connect(void);

/* -------------------- BME events primitives -------------------- */

/**
 * Open events interface
 *
 * @param emask requested events mask
 *
 * @return	events handler
 */
int32_t bmeipc_eopen(int32_t emask);

/**
 * Close events interface
 *
 * @param eh events handler
 *
 * @return	0 on success, -1 on error
 */
int32_t bmeipc_eclose(int32_t eh);

/**
 * Read events
 *
 * @param eh events handler
 *
 * @return	events mask
 */
int32_t bmeipc_eread(int32_t eh);

/**
 * Poll events
 *
 * @param eh events handler
 * @param tout polling timeout
 *
 * @return	received events mask
 */
int32_t bmeipc_epoll(int32_t eh, int32_t tout);

/* -------------------- BME messaging primitives -------------------- */

/** _deprecated__, use bmeipc_recv() instead */
int32_t bme_read(void *msg, int32_t bytes);

/* _deprecated_, use bmeipc_send() instead */
int32_t bme_write(const void *msg, int32_t bytes);

/**
 * Send message to the server and get reply
 *
 * @param sd ipc descriptor
 * @param msg1 [in] request
 * @param len1 request size
 * @param msg2 [out] reply
 * @param len2 reply size
 *
 * @return greater or equal to zero on success,
 * -1 on error
 *
 * @credential  bme::BatteryControl  Needed for all non read-only messages
 *
 * @attention if rmsg is NULL reply status is returned
 *
 * @ingroup bmeipc
 */
int32_t bmeipc_query(int32_t sd, const void *msg1, int32_t len1,
		void *msg2, int32_t len2);
/** _deprecated_ non-reentrant variant
 *
 * @param smsg address of a message to send
 * @param sbytes size of message to send
 * @param rmsg address of a reply buffer
 * @param rbytes size of reply buffer
 * @rbytes_act:	actual size of reply got from the server
 *
 * @return	>= 0 on success, -1 on error
 *		NB: if rmsg is NULL reply status is returned
 */
int32_t bme_send_get_reply(const void *smsg, int32_t sbytes,
		void *rmsg, int32_t rbytes, int32_t *rbytes_act);

/**
 * Send reply message with status, if message is not given only status is sent.
 *
 * @param sd socket descriptor
 * @param st reply status
 * @param msg message address
 * @param bytes message size
 *
 * @return	0 on success, -1 on error
 *
 * @example1	send status only:
 *			bmeipc_reply(sd, st, NULL, -1)
 * @example2	send status with message:
 *			bmeipc_reply(sd, st, msg, sizeof(msg))
 */
int32_t bmeipc_reply(int32_t sd, int32_t st, void *msg, int32_t bytes);

/**
 *
 * function to receive protocol status before reading server response
 * 
 * bmeipc communication consists of the next steps:
 *
 * @li client sends request message to the server
 *
 * @li server sends protocol status back (signed integer, 4 bytes) -
 * negative is error
 *
 * @li server sends reply message back to the client
 * 
 * @param sd bmeipc socket descriptor (>=0)
 * 
 * @return protocol status - <0 is error, >=0 is success
 *
 * @example Client-side communication sequence
 *
 * int rc;
 * rc = bmeipc_send(sd, &msg, sizeof(msg));
 * // check (rc == sizeof(msg))
 * rc = bmeipc_recv_status(sd);
 * if (rc >= 0) {
 *     rc = bmeipc_recv(sd, &reply, sizeof(reply));
 *     // check (rc >= 0)
 * }
 *
 */
int bmeipc_recv_status(int32_t sd);

/**
 * Get a PID of BME server.
 *
 * @param sd socket descriptor
 *
 * @return	PID on success, -1 on error
 */
int32_t bmeipc_getpid(int32_t sd);
/* _deprecated_ non-reentrant variant */
int32_t bme_get_server_pid(void);

/**
 * Read BME cookie
 *
 * @param sd socket descriptor to receive cookie from
 *
 * @return	0 on success, -1 on error
 */
int32_t bmeipc_getcookie(int32_t sd);

/**
 * Write BME cookie
 *
 * @param sd socket descriptor to send cookie to
 *
 * @return	0 on success, -1 on error
 */
int32_t bmeipc_putcookie(int32_t sd);

#endif /* __BMEIPC_H__ */
