/*
 * @file      bmemsg.h: BME mqueue messaging definitions
 *
 * @author    <mikko.k.ylinen at nokia.com>
 *
 * @modified  <aliaksei.katovich at nokia.com>
 *
 * @date      Created: Thu Jul 30 09:19:59 EEST 2009
 *
 * Copyright (C) 2009 Nokia. All rights reserved.
 *
 */

#ifndef __BMEMSG_H__
#define __BMEMSG_H__

#include <bme/bmeipc.h>

#include <stdint.h>
#include <stddef.h>

#pragma pack(push)
#pragma pack(4)

/* System message codes (not related to battery management) */
enum bme_sysmsg_e {
	BME_SYSMSG_GETPID = 0x8000,	/* beyond ISI reuests range */
	BME_SYSMSG_PROXY_OPEN,
	BME_SYSMSG_PROXY_CLOSE,
	BME_SYSMSG_GETSTAT,		/* 0x8003 get em statistics */
};

/*
 * Additional (non-system) message codes used by BME.
 * They must not overlap with constants from em_isi.h
 */
#define BME_AUXMSG_TEST_MODE_CTRL	0x40	/* Test mode control */
#define BME_AUXMSG_FTD_REQ		0x41	/* FTD information request */
#define BME_AUXMSG_FTD_BULK_REQ		0x42	/* FTD bulk information request */
#define BME_AUXMSG_FTD_BULK_REQ2	0x43	/* FTD bulk information request2 */
#define BME_AUXMSG_FTD_BULK_REQ3	0x44	/* FTD bulk information request3 */
#define BME_AUXMSG_LED_CTRL		0x45	/* LED control */
#define BME_AUXMSG_CHARGE_CTRL		0x48	/* charge control */
#define BME_AUXMSG_ISI			0x49    /**< ISI messages transfer */

/* Test mode on/off */
#define ACC_TEST_MODE_OFF		0
#define ACC_TEST_MODE_ON		1

/* -------------------- External messaging  -------------------- */

#include "bme/bmeisa.h"

#ifndef COMPILE_TIME_ASSERT
#define COMPILE_TIME_ASSERT(condition, error_name)	\
	typedef char ERROR_##error_name			\
	[1 / (!!(condition))]
#endif

/*::::::: Client request and reply messages :::::::*/

/** @addtogroup bmeipc */
/*@{ */

/* Measurement action request element */
struct emsg_measurement_req_elem {
	uint8_t type;
	uint8_t period;
};

/**
 * Measurement action
 */
struct emsg_measurement_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t measurement_action;
	uint8_t channel_count;
	struct emsg_measurement_req_elem data[0];
};

COMPILE_TIME_ASSERT(sizeof(struct emsg_measurement_req) == 12
		    , packing_rules_are_incompatible_with_bmeipc_abi);

COMPILE_TIME_ASSERT(offsetof(struct emsg_measurement_req, data) == 9
		    , alignment_rules_are_incompatible_with_bmeipc_abi);

/**
 * Charger info request
*/
struct emsg_charger_info_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
};

/**
 * Charger info reply
 */
struct emsg_charger_info_reply {
	uint8_t  type;
	uint8_t  status;
	uint32_t zero;
	uint32_t flags;
	uint16_t voltage;
	uint16_t current;
	uint16_t reserved;
};

union emsg_charger_info {
	struct emsg_charger_info_req request;
	struct emsg_charger_info_reply reply;
};

/**
 * Control write
 */
struct emsg_control_write_req {
	uint16_t type;
	uint16_t subtype;
	bmeisa_cha_ctrl_info_t ctrl;
};

/**
 * Control read
 */
struct emsg_control_read_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
	bmeisa_cha_ctrl_info_t ctrl;
};

/**
 * Control read
*/
struct emsg_control_read_reply {
	uint32_t zero;
	bmeisa_cha_ctrl_info_t ctrl;
};

union emsg_control_read {
	struct emsg_control_read_req request;
	struct emsg_control_read_reply reply;
};

/**
 * Battery info request
*/
struct emsg_battery_info_req {
	uint16_t type; /**< EM_BATTERY_INFO_REQ */
	uint16_t subtype;
	uint32_t flags;
};

/**
 * Battery info reply
*/
struct emsg_battery_info_reply {
	uint32_t zero;
	uint32_t flags;
	uint16_t state;
	uint16_t bat_type;
	uint16_t nominal_capa;
	uint16_t temp;
	uint16_t voltage;
	uint16_t voltage_tx_on;
	uint16_t voltage_tx_off;
	uint16_t voltage_pwm_on;
	uint16_t voltage_pwm_off;
	uint16_t generation;
	uint16_t voltage_sh_chk;
};

union emsg_battery_info {
	struct emsg_battery_info_req request;
	struct emsg_battery_info_reply reply;
};

/**
 * Charging info
 */
struct emsg_charging_info_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
};

/**
 * Charging info
*/
struct emsg_charging_info_reply {
	uint32_t zero;
	uint32_t flags;
	uint8_t state;
	uint8_t mode;
	uint16_t reserved;
};

union emsg_charging_info {
	struct emsg_charging_info_req request;
	struct emsg_charging_info_reply reply;
};

/**
 * Monitoring info
 */
struct emsg_monitoring_info_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
};

/**
 * Monitoring info
*/
struct emsg_monitoring_info_reply {
	uint32_t zero;
	uint32_t flags;

	/** The battery power state (see enum bmeipc_battery_state_e) */
	uint8_t batt_power;

	/** Remaining capacity as percentage (0-100) */
	uint8_t batt_capa_precentage;

	/** The maximum number of bars */
	uint8_t batt_max_num_bars;

	/** The current number of bars (0 - batt_max_num_bars) */
	uint8_t batt_bars;

	/** The Remaining capacity (mAh) */
	uint16_t batt_capa;

	uint16_t batt_time;

	/** Not supported */
	uint8_t batt_max_audio_bars;

	/** Not supported */
	uint8_t batt_audio_bars;

	uint16_t reserved2;
};

union emsg_monitoring_info {
	struct emsg_monitoring_info_req request;
	struct emsg_monitoring_info_reply reply;
};

/**
 * Usetime info
 */
struct emsg_usetime_info_req {
	uint16_t type;      /**< EM_BATTERY_USETIME_REQ */
	uint16_t subtype;
	uint16_t current;   /** The expected average current (mA) */
	uint16_t reserved;
};

struct emsg_usetime_info_reply {
	uint32_t zero;
	uint32_t flags;

	/** The remaining use time (seconds) for the specified current */
	uint32_t time;

	uint32_t reserved;
};

union emsg_usetime_info {
	struct emsg_usetime_info_req request;
	struct emsg_usetime_info_reply reply;
};

/**
 * Set monitoring voltage
 */
struct emsg_monitoring_voltage_req {
	uint16_t type;
	uint16_t subtype;
	uint16_t voltage_level;
	uint16_t reserved;
};

/**
 * ADC read
 */
struct emsg_adc_read_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t channel;
	uint8_t num_reads;
	uint16_t reserved;
};

/**
 * ADC read
*/
struct emsg_adc_read_reply {
	uint32_t zero;
	int32_t result;
};

union emsg_adc_read {
	struct emsg_adc_read_req request;
	struct emsg_adc_read_reply reply;
};

/**
 * ADC unit read
 */
struct emsg_adc_unit_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t channel;
	uint8_t num_reads;
	uint16_t reserved;
};

/**
 * ADC unit read
*/
struct emsg_adc_unit_reply {
	uint32_t zero;
	int32_t result;
};

union emsg_adc_unit {
	struct emsg_adc_unit_req request;
	struct emsg_adc_unit_reply reply;
};

/**
 * Write ADC calibration data
 */
struct emsg_adc_cal_write_req {
	/* Fixed part, must be sent first and atomically */
	uint16_t type;
	uint16_t subtype;
	uint8_t num_data;
	uint8_t action;
	/* num_data elements follow */
};

/**
 * Read ADC calibration data
 */
struct emsg_adc_cal_read_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t action;
	/** The channel number may be also one of "pseudo-channel" IDs:
	   EM_ADC_MAIN_CAL (to get calibration data for AD converter), or
	   EM_ADC_ALL_CAL (to get calibration data for all channels */
	uint8_t channel;
};

/* Structure of 'ADC channel calibration data' element */
struct adc_chan_cal_data_elem {
	bmeisa_adc_cal_data_str_t cdata;
	uint8_t alg;
	uint8_t reserved;
	uint16_t parameter1;	/* Usually reference_voltage */
};

/**
 * Read ADC calibration data
*/
struct emsg_adc_cal_read_reply {
	uint8_t num_data;
	/* num_data elements follow */
	struct adc_chan_cal_data_elem accd[0];
};

union emsg_adc_cal_read {
	struct emsg_adc_cal_read_req request;
	struct emsg_adc_cal_read_reply reply;
};

/**
 * PSM feature
 */
struct emsg_psm_feature_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t state;
};

/**
 * PSM read
 */
struct emsg_psm_read_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
};

/**
 * PSM read
*/
struct emsg_psm_read_reply {
	uint32_t zero;
	uint32_t flags;
	uint16_t bat_current;
	uint16_t phone_current;
	uint16_t discharge_amount;
	uint8_t cs_act_state;
	uint8_t cs_act_protocol;
	uint8_t cs_idle_activity;
	uint8_t reserved;
	uint16_t batmon4_phone_state;
	uint16_t feature_counter;
	uint32_t activity;
};

union emsg_psm_read {
	struct emsg_psm_read_req request;
	struct emsg_psm_read_reply reply;
};

/**
 * PSM internal write
 */
struct emsg_psm_internal_write_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t device;
	uint8_t activity_level;
};

#if (EM_MON_ADC_LOCK_SUPPORT == TRUE)
/**
 * Set/lock ADC unit calibration data
 */
struct emsg_srv_adc_unit_lock_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t	channel;
	uint8_t action;
	int16_t value;
};

#endif

#if (EM_FEA_FAST_ADC_ALIGNMENT == TRUE)

/* ADC tuning request element */
struct adc_tune_req_elem {
	uint8_t chan_id;
	uint8_t num_samples;
	int16_t trigger_level;
};

/**
 * Start ADC tuning
 */
struct emsg_adc_tune_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t reserved;
	uint8_t num_states;
	uint16_t state_trans_timer_val;
	uint16_t timeout_timer_val;
	/* num_states elements follow */
	struct adc_tune_req_elem reqarr[0];
};

/* ADC tuning reply element */
struct adc_tune_reply_elem {
	uint16_t status;
	uint16_t chan_id;
	int16_t meas_val;
};

/**
 * ADC tuning
*/
struct emsg_adc_tune_reply {
	uint8_t num_blocks;
	/* num_blocks elements follow */
	struct adc_tune_reply_elem replyarr[0];
};

union emsg_adc_tune {
	struct emsg_adc_tune_req request;
	struct emsg_adc_tune_reply reply;
};

#endif

#if (EM_FEA_THERMOMETER_ENABLED == TRUE)

/**
 * IPC Temperature request
*/
struct emsg_thermo_req {
	uint16_t type; /**< EM_TEMPERATURE_QUERY_REQ */
	uint16_t subtype;
};

/**
 * IPC Temperature reply
*/
struct emsg_thermo_reply {
	uint32_t zero;
	uint32_t result; /**< temperature, K */
	uint8_t status;
	uint8_t reserved[3];
};

union emsg_thermo {
	struct emsg_thermo_req request;
	struct emsg_thermo_reply reply;
};
#endif

#if (EM_FEA_POWER_MANAGEMENT_ENABLED == TRUE)
/**
 * Power
 */
struct emsg_power_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t consumer;
	uint8_t state;
	uint16_t reserved;
};

/**
 * Power
*/
struct emsg_power_reply {
	uint32_t zero;
	uint8_t status;
	uint8_t reserved[3];
};

union emsg_power {
	struct emsg_power_req request;
	struct emsg_power_reply reply;
};
#endif

/*@}*/

#if (EM_FTD_ENABLE == TRUE)

/**
 * Flags for EM FTD bulk data
 */
#define EM_FTD_REQ_FLAG_STATUS			1
#define EM_FTD_REQ_FLAG_INST_BAT_VOLT		2
#define EM_FTD_REQ_FLAG_REMAIN_STBY_MINUTES	4
#define EM_FTD_REQ_FLAG_NUMBER_BAT_LOW_WARN	8
#define EM_FTD_REQ_FLAG_MISC_MON_INT_FLAGS1	0x10
#define EM_FTD_REQ_FLAG_CALC_VOLT_LEVEL		0x20
#define EM_FTD_REQ_FLAG_WARN_COUNT		0x40
#define EM_FTD_REQ_FLAG_DMF_BAT_VOLT		0x80
#define EM_FTD_REQ_FLAG_BAT_BOOT_VOLT		0x100
#define EM_FTD_REQ_FLAG_CALC_BAT_BAR		0x200
#define EM_FTD_REQ_FLAG_VOLT_AT_FIRST_BAT_LOW	0x400
#define EM_FTD_REQ_FLAG_CURRENT_MON_EST1	0x800
#define EM_FTD_REQ_FLAG_CHARGED_AMOUNT		0x1000
#define EM_FTD_REQ_FLAG_POWEROFF_LOG_RESERVED	0x2000	/* Not supported yet */
#define EM_FTD_REQ_FLAG_LOWEST_V_TXOFF		0x4000
#define EM_FTD_REQ_FLAG_LOWEST_V_TXON		0x8000
#define EM_FTD_REQ_FLAG_LARGEST_DV		0x10000
#define EM_FTD_REQ_FLAG_BATT_BAR_LOG_MASK	0x20000
#define EM_FTD_REQ_FLAG_PREV_BATT_BAR_LEV	0x40000
#define EM_FTD_REQ_FLAG_BATT_LOW_REASON		0x80000
/* Added 21.6.05 */
#define EM_FTD_REQ_FLAG_CS_STATE		0x100000
#define EM_FTD_REQ_FLAG_NUM_OF_BAT_BARS		0x200000
#define EM_FTD_REQ_FLAG_BAT_TYPE		0x400000
#define EM_FTD_REQ_FLAG_CURRENT_BAT_TEMP	0x800000
#define EM_FTD_REQ_FLAG_BAT_CAPACITY		0x1000000
#define EM_FTD_REQ_FLAG_BAT_IMP			0x2000000
#define EM_FTD_REQ_FLAG_BAT_FULL_THRES		0x4000000
#define EM_FTD_REQ_FLAG_BAT_LOW_THRES		0x8000000
#define EM_FTD_REQ_FLAG_MON_ALG_ADJ		0x10000000
#define EM_FTD_REQ_FLAG_PARAM_ERRORS		0x20000000
#define EM_FTD_REQ_FLAG_CURRENT_MON_EST2	0x40000000

/* Flags for EM FTD bulk request2 */
#define EM_FTD_REQ2_FLAG_EST_TX_OFF		1
#define EM_FTD_REQ2_FLAG_EST_TX_ON		2
#define EM_FTD_REQ2_FLAG_BAT_POW_IND		4
#define EM_FTD_REQ2_FLAG_MISC_MON_INT_FLAGS2	8
#define EM_FTD_REQ2_FLAG_MISC_MON_INT_FLAGS3	0x10
#define EM_FTD_REQ2_FLAG_MON_VOLT		0x20
#define EM_FTD_REQ2_FLAG_ELAPSED_TIME		0x40
#define EM_FTD_REQ2_FLAG_PHI_VARIABLE		0x80
#define EM_FTD_REQ2_FLAG_DELTA_PHI_VARIABLE	0x100
#define EM_FTD_REQ2_FLAG_CHA_METHOD		0x200
#define EM_FTD_REQ2_FLAG_PRESENT_CHA_MODE	0x400
#define EM_FTD_REQ2_FLAG_PREVIOUS_CHA_MODE	0x800
#define EM_FTD_REQ2_FLAG_PRESENT_CHR_TYPE	0x1000
#define EM_FTD_REQ2_FLAG_PREVIOUS_CHR_TYPE	0x2000
#define EM_FTD_REQ2_FLAG_CHA_INST_BAT_VOLT	0x4000
#define EM_FTD_REQ2_FLAG_NUMBER_CHR_CHECKS	0x8000
#define EM_FTD_REQ2_FLAG_CHR_REC_STATE		0x10000
#define EM_FTD_REQ2_FLAG_INST_CHR_CURRENT	0x20000
#define EM_FTD_REQ2_FLAG_BAT_FULL		0x40000
#define EM_FTD_REQ2_FLAG_CHA_TIME		0x80000
#define EM_FTD_REQ2_FLAG_AVERAGE_CHR_VOLT	0x100000
#define EM_FTD_REQ2_FLAG_CALC_CHR_CURRENT	0x200000
#define EM_FTD_REQ2_FLAG_HW_PWM_VALUE		0x400000
#define EM_FTD_REQ2_FLAG_SW_PWM_VALUE		0x800000
#define EM_FTD_REQ2_FLAG_OS_PWM_BAT_VOLT	0x1000000
#define EM_FTD_REQ2_FLAG_CS_PWM_BAT_VOLT	0x2000000

/* Flags for EM FTD bulk request3 */
#define	EM_FTD_REQ3_FLAG_CONF_BAT_FOOTPRINT	1
#define	EM_FTD_REQ3_FLAG_CONF_BAT_BARS		2
#define	EM_FTD_REQ3_FLAG_CONF_MIN_CURRENT	4
#define	EM_FTD_REQ3_FLAG_CONF_BAT_SAFETY_LEVEL	8
#define	EM_FTD_REQ3_FLAG_CONF_BAT_EMPTY_LEVEL	0x10

/** @addtogroup bmeipc */
/*@{ */

/**
 * FTD information
 */
struct emsg_ftd_data_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
};

/**
 * FTD information
*/
struct emsg_ftd_data_reply {
	uint32_t zero;
	uint32_t flags_reserved;
	uint32_t result;
};

/**
 * FTD bulk information data
*/
struct emsg_ftd_bulk_data_reply {
	uint32_t zero;
	uint32_t flags;
	uint8_t status;
	uint16_t instant_v_bat;
	uint32_t remain_stby_mins;
	uint16_t no_bat_low_warnings;
	uint8_t misc_flags_mask1;
	uint16_t check_v_bat;
	uint16_t warn_interval_cnt;
	uint16_t dmf_out_voltage;
	uint16_t bm4_init_v_bat;
	uint16_t time_per_bar_min;
	uint16_t v_bat_1st_bat_low;
	uint32_t avg_phone_curr_ua;
	int16_t last_cha_amount;
	uint16_t lowest_v_txoff;
	uint16_t lowest_v_txon;
	int16_t largest_dv;
	uint8_t batt_bar_log_mask;
	uint8_t prev_batt_bar_lev;
	uint8_t batt_low_reason;
	/* Added 21.6.05 */
	uint8_t cs_state;
	uint8_t num_bat_bars;
	uint16_t bat_type;
	int16_t bat_temp;
	uint16_t bat_capacity;
	uint16_t bat_imp;
	uint16_t bat_full_thres;
	uint16_t bat_low_thres;
	uint16_t mon_alg_adj;
	uint16_t mon_param_errors;
	uint32_t curr_mon_est2;
};

/**
 * FTD bulk information data2
*/
struct emsg_ftd_bulk_data2_reply {
	uint32_t zero;
	uint32_t flags;
	uint32_t elapsed_time;
	uint16_t txoff_volt;
	uint16_t txon_volt;
	uint8_t power_state;
	uint8_t mon_flags2;
	uint8_t mon_flags3;
	uint8_t cha_method;
	uint16_t last_v_model_mv;
	uint16_t phi0_mv;
	int16_t delta_phi0_mv;
	uint8_t cha_mode;
	uint8_t prev_cha_mode;
	uint8_t chr_type;
	uint8_t prev_chr_type;
	int16_t cha_inst_bat_volt;
	uint8_t num_chr_checks;
	uint8_t chr_rec_state;
	uint32_t inst_chr_current;
	uint16_t cha_time;
	uint16_t avg_chr_volt;
	uint16_t chr_current;
	uint8_t bat_full;
	uint8_t hw_pwm_val;
	uint8_t sw_pwm_val;
	uint8_t reserved1;
	int16_t vbat_osv;
	int16_t vbat_csv;
};

/**
 * FTD bulk information data3
*/
struct emsg_ftd_bulk_data3_reply {
	uint32_t zero;
	uint32_t flags;
	uint16_t conf_bat_footprint;
	uint16_t conf_min_current;
	uint16_t conf_bat_safety_level;
	uint16_t conf_bat_empty_level;
	uint8_t conf_bat_bars;
};

union emsg_ftd_data {
	struct emsg_ftd_data_req request;
	struct emsg_ftd_data_reply reply;
};

union emsg_ftd_bulk_data {
	struct emsg_ftd_data_req request;
	struct emsg_ftd_bulk_data_reply reply;
	struct emsg_ftd_bulk_data2_reply reply2;
	struct emsg_ftd_bulk_data3_reply reply3;
};

/* -------------------- FTD information routines -------------------- */

/**
 * Get EM FTD data.
 *
 * @subtype: message subtype (EM_SB_FTD_*)
 *
 * @return positive value if successful, -1 otherwise
 */
int32_t bmeipc_getftd(uint16_t subtype);

/**
 * Get EM FTD bulk data.
 *
 * @flags: EM_FTD_REQ_FLAG_* flags
 * @reply: address of reply buffer
 *
 * @return 0 if successful, -1 otherwise
 */
int32_t bmeipc_getftdbulk(uint32_t flags, struct emsg_ftd_bulk_data_reply *rep);

#endif

/**
 * Test mode
 */
struct auxmsg_test_mode_req {
	uint16_t type;
	uint16_t subtype;
	uint8_t onoff;
};

/**
 * USB charge mode
 */
struct auxmsg_usb_charge_mode_req {
	uint16_t type;
	uint16_t subtype;
	uint32_t mode;
	uint8_t reserved[3];
};

struct emsg_isi {
	uint16_t type;
	uint16_t subtype;
	uint16_t req_id;
	uint16_t data_len;
} __attribute__((packed));

struct emsg_isi_reply {
	uint16_t len;
	uint16_t count;
};

/* This union combines all possible messages */
union bme_ext_messages {
	union emsg_charger_info charger_info;
	union emsg_charging_info charging_info;
	union emsg_battery_info battery_info;
	union emsg_monitoring_info monitoring_info;
	union emsg_usetime_info usetime_info;
	struct emsg_monitoring_voltage_req set_monitoring_voltage;
	union emsg_control_read ctrl_read;
	struct emsg_control_write_req ctrl_write;
	union emsg_adc_read adc_raw_read;
	union emsg_adc_unit adc_unit_read;
	union emsg_adc_cal_read adc_cal_read;
	struct emsg_adc_cal_write_req adc_cal_write;
	struct emsg_psm_feature_req psm_feature;
	union emsg_psm_read psm_read;
	struct emsg_psm_internal_write_req psm_write;
#if (EM_MON_ADC_LOCK_SUPPORT == TRUE)
	struct emsg_srv_adc_unit_lock_req adc_unit_lock;
#endif
#if (EM_FEA_FAST_ADC_ALIGNMENT == TRUE)
	union emsg_adc_tune adc_tune;
#endif
#if (EM_FEA_POWER_MANAGEMENT_ENABLED == TRUE)
	union emsg_power power;
#endif
#if (EM_FEA_THERMOMETER_ENABLED == TRUE)
	union emsg_thermo thermo;
#endif
#if (EM_FTD_ENABLE == TRUE)
	union emsg_ftd_bulk_data ftd_data;
#endif
	struct auxmsg_test_mode_req aux_test;
	struct emsg_measurement_req measurement;
	struct auxmsg_usb_charge_mode_req usb_charge_mode;
	struct emsg_isi isi;
};

COMPILE_TIME_ASSERT(BME_IPC_MSG_MAX_SIZE > sizeof(union bme_ext_messages)
		    , max_allowed_msg_size_exceeds_some_msg_size);

/* -------------------- Indication messages -------------------- */

/* EM info indication */
struct emsg_info_ind {
	uint16_t type;
	uint16_t subtype;
	uint32_t flags;
	uint8_t charger_status;
	uint8_t charging_state;
	uint8_t charging_mode;
	uint8_t batt_max_num_bars;
	uint8_t batt_bars_data;
	uint8_t batt_power_data;
	uint16_t batt_idletime;
	uint16_t batt_usetime;
};

/* EM monitoring voltage indication */
struct emsg_mon_voltage_ind {
	uint16_t type;
	uint16_t subtype;
	uint16_t voltage;
	uint16_t reserved;
};

/* Temperature indication */
struct emsg_thermo_ind {
	uint16_t type;
	uint16_t subtype;
	uint32_t result;
	uint8_t status;
	uint8_t reserved[3];
};

/*@}*/

#undef COMPILE_TIME_ASSERT

#pragma pack(pop)

#endif /* __BMEMSG_H__ */
